ID$ = "ascii JTAG 3" + Chr$(0)       ' 3 -> 20k serial input buffer
Option autorun ON
CPU 48
Option baudrate 230400               ' slow = 38400, fast = 230400
Open "COM1: 230400, 20000" As #5     ' used for buffered input only

For B = 2 To 7: Pin(B) = 0: Next B

SetPin 2, OOUT         ' MCLR, connect to pin 1 on target
SetPin 3, DOUT         ' TMS, connect to pins 14 and 22 on target
SetPin 4, DOUT         ' TDI, connect to pin 16 on target
SetPin 5, DIN          ' TDO, connect to pin 18 on target
SetPin 6, DOUT         ' CLK, connect to pin 17 on target
SetPin 7, DOUT         ' status LED (via 1k resistor to ground)

' the following code needs to be fast

start:
  B = Asc(Input$(1, #5))
  If (B = 0) GoTo start
  If (B > &h40) Then
    Port(3,2) = (B And 3)
    If (B < &h60) Then Print Chr$(&h30 + Pin(5));
    Pulse 6, 0.005
    GoTo start
  EndIf
  If B=Asc(".") Then GoTo start
  If B=Asc(">") Then Print "<";: GoTo start

' the following code can be slow

  Port(3,2) = 0                      ' set TDI and TMS low (safety)

  If B = Asc("2") Then Pin(2) = 0    ' set MCLR low (target in reset)
  If B = Asc("3") Then Pin(2) = 1    ' set MCLR high (target running)

  If B = Asc("4") Then Pin(7) = 0    ' turn off LED/power
  If B = Asc("5") Then Pin(7) = 1    ' turn on LED/power

  If B = Asc("8") Then Pause 10      ' 10mS (short) delay
  If B = Asc("?") Then Print ID$;    ' generate ID
  If B = Asc("*") Then GoTo unlock
GoTo start


' ICSP erase to unlock devices with JTAG_ENABLE = 0

unlock:

Pin(25) = 0            ' PGD (data)   --> pin 4 on target
Pin(24) = 0            ' PGC (clock)  --> pin 5 on target
SetPin 25, DOUT
SetPin 24, DOUT

Pin(2) = 0             ' hold in reset
Pin(7) = 0             ' remove power
Pause 100
Pin(7) = 1             ' reapply power

Pulse 2, 0.200         ' pulse MCLR high for 200uS
For I = 1 To 32        ' send "MCHP" signature
  Read D
  Pin(25) = D
  Pulse 24, 0.005
Next I
Pin(2) = 1             ' release reset
Pause 100

Read D                 ' send "erase chip" command sequence
Do
  Pin(25) = D \ 2      ' TDI
  Pulse 24, 0.005
  Pin(25) = D And 1    ' TMS
  Pulse 24, 0.005
  SetPin 25, DIN
  Pulse 24, 0.005      ' TDO (ignored)
  Pulse 24, 0.005
  SetPin 25, DOUT
  Read D
  If D = 9 Then Pause 100: Read D
Loop Until (D = 99)

Pause 400              ' in theory, target is now unlocked
Pin(7) = 0             ' remove power from target
Pause 100
Pin(7) = 1             ' reapply power, see if still alive
SetPin 25, OFF
SetPin 24, OFF
Restore
GoTo start


Data 0,1,0,0, 1,1,0,1,  0,1,0,0, 0,0,1,1
Data 0,1,0,0, 1,0,0,0,  0,1,0,1, 0,0,0,0       ' "MCHP"

Data 1,1,1,1,1,0,                       9      ' run_test/idle

Data 1,  1,0,0,  0,0,2,0,1,        1,0         ' TAP_SW_MTAP
Data 1,  1,0,0,  2,2,2,0,1,        1,0         ' MTAP_COMMAND
Data     1,0,0,  0,0,2,2,2,2,2,3,  1,0, 9      ' MCHP_ERASE

Data     1,0,0,  0,0,0,0,0,0,0,1,  1,0         ' MCHP_STATUS

Data 99

